@@ -1,5 +1,7 @@
 import os
 
+from quickfind.Searcher import CString
+
 def truncate_front(line, length=70):
     reduce_amt = len(line) - length
     # If it already fits
@@ -30,3 +32,14 @@ def rec_dir_up(dir):
             if newdir == dir: break
             dir = newdir
 
+def highlight(v, query, color="green"):
+    i = v.lower().rfind(query.lower())
+    if query and i > -1:
+        left = v[:i]
+        lq = len(query)
+        highlight = v[i:i+lq]
+        right = v[i+lq:]
+        return [ left, CString(highlight, color), right ]
+
+    return [v]
+
